/*
 *  linux/fs/supermount/mediactl.c
 *
 *  Original version:
 *      Copyright (C) 1995, 1997
 *      Stephen Tweedie (sct@dcs.ed.ac.uk)
 *
 *  Rewriten for kernel 2.2, 2.4. (C) 1999, 2000 Alexis Mikhailov
 *                                    (alexis@abc.cap.ru)
 *  Rewritten for kernel 2.4.21 (C) 2003 Andrey Borzenkov
 *                                       (arvidjaar@mail.ru)
 *
 */

#define S_DBG_TRACE_CURRENT S_DBG_TRACE_MEDIACTL
#include "supermount.h"

/* 
 * Try to lock the drive door.  This is not guaranteed to work on all
 * hardware, but we try any tricks we know of anyway.  
 */
void
supermount_handle_door(struct super_block *sb, int operation)
{
	kdev_t dev;
	int major;
	const struct block_device_operations *fops;
	struct super_block *subsb;

	if (!subfs_is_active(sb))
		return;

	subsb = subfs_sb(sb);

	dev = subsb->s_dev;
	major = MAJOR(dev);
	if (major >= MAX_BLKDEV || !(fops = get_blkfops(major)))
		return;
	if (!fops->mediactl)
		return;
	/*
	 * tray is (un-)locked in open (BKL for bdev), release (BKL for bdev)
	 * and ioctl (BKL). We are in good company.
	 * This must be changed if block devices ever stop using BKL
	 * for open/release. Unfortunately, using just bdev->bd_sem is not
	 * enough due to ioctl.
	 */
	lock_kernel();
	fops->mediactl(dev, operation, 0);
	unlock_kernel();
}
