/*
 *  linux/fs/supermount/utils.c
 *
 *  Original version:
 *      Copyright (C) 2003
 *      Andrey Borzenkov (arvidjaar@mail.ru)
 */

#define S_DBG_TRACE_CURRENT S_DBG_TRACE_UTILS
#include "supermount.h"

int
supermount_init_dentry_info(struct dentry *dentry)
{
	struct super_block *sb = dentry->d_sb;
	struct supermount_dentry_info *sdi;
	int rc;

	SUPERMOUNT_TRACE_ENTER(sb);

	rc = -ENOMEM;
	sdi = kmalloc(sizeof(*sdi), GFP_KERNEL);
	if (!sdi)
		goto out;

	memset(sdi, 0, sizeof(*sdi));

	INIT_LIST_HEAD(&sdi->list);
	sdi->generation = supermount_sb_generation(sb);
	sdi->dentry = 0;
	sdi->host = dentry;
	
	dentry->d_fsdata = sdi;
	dentry->d_op = &supermount_dops;

	rc = 0;
out:
	SUPERMOUNT_TRACE_LEAVE(sb);

	return rc;

}

void
supermount_attach_subdent(struct dentry *dentry, struct dentry *subdent)
{
	struct super_block *sb = dentry->d_sb;
	struct supermount_sb_info *sbi = supermount_sbi(sb);
	struct supermount_dentry_info *sdi;

	SUPERMOUNT_TRACE_ENTER(sb);

	sdi = supermount_d(dentry);
	sdi->dentry = dget(subdent);
	list_add(&sdi->list, &sbi->s_dentries);

	SUPERMOUNT_TRACE_LEAVE(sb);

}

int
supermount_init_file_info(struct file *file)
{
	struct super_block *sb = file->f_dentry->d_sb;
	struct supermount_file_info *sfi;
	struct supermount_dentry_info *sdi;
	int rc;

	SUPERMOUNT_TRACE_ENTER(sb);

	rc = -ENOMEM;
	sfi = kmalloc(sizeof(*sfi), GFP_KERNEL);
	if (!sfi)
		goto out;

	memset(sfi, 0, sizeof(*sfi));

	sdi = supermount_d(file->f_dentry);

	INIT_LIST_HEAD(&sfi->list);
	sfi->host = file;
	sfi->generation = sdi->generation;
	sfi->owner = current->pid;
	sfi->vm_ops = 0;
	sfi->file = 0;
	
	file->f_supermount = sfi;

	rc = 0;
out:
	SUPERMOUNT_TRACE_LEAVE(sb);

	return rc;

}

int
supermount_attach_subfile(struct file *file, struct file *subfile)
{
	struct super_block *sb = file->f_dentry->d_sb;
	struct supermount_sb_info *sbi = supermount_sbi(sb);
	struct supermount_file_info *sfi;
	int rc;

	SUPERMOUNT_TRACE_ENTER(sb);

	subfs_lock(sb);

	rc = -ENOMEDIUM;
	if (!subfs_is_active(sb))
		goto out;

	rc = -ESTALE;
	if (is_file_obsolete(file))
		goto out;

	sfi = supermount_f(file);
	sfi->file = subfile;
	list_add(&sfi->list, &sbi->s_files);
	rc = 0;

out:
	subfs_unlock(sb);

	SUPERMOUNT_TRACE_LEAVE(sb);

	return rc;
}
